﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace FlowerShopDatabaseImplement.Implements
{
    public class BouquetStorage : IBouquetStorage
    {
        public BouquetViewModel? GetElement(BouquetSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BouquetName) && !model.Id.HasValue)
            {
                return null;
            }

            using var context = new FlowerShopDatabase();

            return context.Bouquets.Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .FirstOrDefault(
                x => (!string.IsNullOrEmpty(model.BouquetName) && x.BouquetName == model.BouquetName) || (model.Id.HasValue && x.Id == model.Id)
            )?.GetViewModel;
        }

        public List<BouquetViewModel> GetFilteredList(BouquetSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BouquetName))
            {
                return new();
            }

            using var context = new FlowerShopDatabase();

            return context.Bouquets.Include(x => x.Components).ThenInclude(x => x.Component).Where(x => x.BouquetName.Contains(model.BouquetName)).Select(x => x.GetViewModel).ToList();
        }

        public List<BouquetViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();
            return context.Bouquets.Include(x => x.Components).ThenInclude(x => x.Component).Select(x => x.GetViewModel).ToList();
        }

        public BouquetViewModel? Insert(BouquetBindingModel model)
        {
            using var context = new FlowerShopDatabase();

            using var transaction = context.Database.BeginTransaction();
            {
                try
                {
                    var newBouquet = Bouquet.Create(context, model);
                    if (newBouquet == null)
                    {
                        transaction.Rollback();
                        return null;
                    }

                    context.Bouquets.Add(newBouquet);

                    context.SaveChanges();
                    context.Database.CommitTransaction();

                    return newBouquet.GetViewModel;
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    return null;
                }
            }
        }

        public BouquetViewModel? Update(BouquetBindingModel model)
        {
            using var context = new FlowerShopDatabase();

            using var transaction = context.Database.BeginTransaction();
            {
                try
                {
                    var bouquet = context.Bouquets.FirstOrDefault(x => x.Id == model.Id);
                    if (bouquet == null)
                    {
                        transaction.Rollback();
                        return null;
                    }

                    bouquet.Update(model);
                    bouquet.UpdateComponents(context, model);

                    context.SaveChanges();
                    context.Database.CommitTransaction();

                    return bouquet.GetViewModel;
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    return null;
                }
            }
        }

        public BouquetViewModel? Delete(BouquetBindingModel model)
        {
            using var context = new FlowerShopDatabase();

            var element = context.Bouquets.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Bouquets.Remove(element);
                context.SaveChanges();

                return element.GetViewModel;
            }

            return null;
        }
    }
}
