﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace FlowerShopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new FlowerShopDatabase();
            return context.Orders.Include(x => x.Bouquet)
                                 .Include(x => x.Client)
                                 .Include(x => x.Implementer)
                                 .FirstOrDefault(x => (model.Statuses == null || model.Statuses != null && model.Statuses.Contains(x.Status)) &&
                                     model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId ||
                                     model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue && model.Statuses == null)
            {
                return new();
            }

            using var context = new FlowerShopDatabase();

            if (model.ClientId.HasValue)
            {
                return context.Orders.Include(x => x.Client).Include(x => x.Implementer).Where(x => x.ClientId == model.ClientId).Select(x => x.GetViewModel).ToList();
            }

            if (model.Statuses != null)
            {
                return context.Orders.Include(x => x.Client).Include(x => x.Implementer).Where(x => model.Statuses.Contains(x.Status)).Select(x => x.GetViewModel).ToList();
            }

            return context.Orders.Include(x => x.Bouquet).Include(x => x.Implementer).Where(x => x.Id == model.Id || model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo).Select(x => x.GetViewModel).ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();
            return context.Orders.Include(x => x.Bouquet).Include(x => x.Implementer).Select(x => x.GetViewModel).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }

            using var context = new FlowerShopDatabase();

            context.Orders.Add(newOrder);
            context.SaveChanges();

            return context.Orders.Include(x => x.Bouquet).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new FlowerShopDatabase();

            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();

            return context.Orders.Include(x => x.Bouquet).Include(x => x.Implementer).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new FlowerShopDatabase();
            
            var element = context.Orders.Include(x => x.Client).Include(x => x.Implementer).FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();

                return element.GetViewModel;
            }

            return null;
        }
    }
}
