﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Enums;
using FlowerShopDataModels.Models;
using System.ComponentModel.DataAnnotations;


namespace FlowerShopDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        [Required]
        public int BouquetId { get; private set; }

        [Required]
        public int ClientId { get; private set; }

        public int? ImplementerId { get; private set; }

        [Required]
        public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public virtual Bouquet Bouquet { get; set; }
        public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order
            {
                BouquetId = model.BouquetId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                Id = model.Id
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel { get {
                var context = new FlowerShopDatabase();
                return new()
                {
                    BouquetId = BouquetId,
                    ClientId = ClientId,
                    ImplementerId = ImplementerId,
                    Count = Count,
                    Sum = Sum,
                    DateCreate = DateCreate,
                    DateImplement = DateImplement,
                    Id = Id,
                    Status = Status,
                    BouquetName = context.Bouquets.FirstOrDefault(x => x.Id == BouquetId)?.BouquetName ?? string.Empty,
                    ClientFIO = context.Clients.FirstOrDefault(x => x.Id == ClientId)?.ClientFIO ?? string.Empty,
                    ImplementerFIO = context.Implementers.FirstOrDefault(x => x.Id == ImplementerId)?.ImplementerFIO ?? string.Empty
                };
            } 
        }
    }
}
