﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopFileImplement.Models;

namespace FlowerShopFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;

        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (model.ImplementerId.HasValue && model.Statuses != null)
            {
                return AttachInfo(source.Orders.FirstOrDefault(x => x.ImplementerId == model.ImplementerId && model.Statuses.Contains(x.Status))?.GetViewModel);
            }
            
            if (model.ImplementerId.HasValue)
            {
                return AttachInfo(source.Orders.FirstOrDefault(x => x.ImplementerId == model.ImplementerId)?.GetViewModel);
            }

            if (!model.Id.HasValue)
            {
                return null;
            }

            return AttachInfo(source.Orders.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel);
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue && model.Statuses == null)
            {
                return new();
            }

            if (model.ClientId.HasValue)
            {
                return source.Orders.Where(x => x.ClientId == model.ClientId).Select(x => AttachInfo(x.GetViewModel)).ToList();
            }

            if (model.Statuses != null)
            {
                return source.Orders.Where(x => model.Statuses.Contains(x.Status)).Select(x => AttachInfo(x.GetViewModel)).ToList();
            }

            return source.Orders.Where(x => x.Id == model.Id || model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo).Select(x => AttachInfo(x.GetViewModel)).ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => AttachInfo(x.GetViewModel)).ToList();
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            order.Update(model);
            source.SaveOrders();

            return AttachInfo(order.GetViewModel);
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            source.Orders.Add(newOrder);
            source.SaveOrders();

            return AttachInfo(newOrder.GetViewModel);
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            source.Orders.Remove(order);
            source.SaveOrders();

            return AttachInfo(order.GetViewModel);
        }

        private OrderViewModel AttachInfo(OrderViewModel? model)
        {
            if (model == null)
            {
                return null;
            }

            var bouquet = source.Bouquets.Find(b => b.Id == model.BouquetId);
            model.BouquetName = bouquet is null ? String.Empty : bouquet.BouquetName;

            var client = source.Clients.FirstOrDefault(x => x.Id == model.ClientId);
            if (client != null)
            {
                model.ClientFIO = client.ClientFIO;
            }

            return model;
        }
    }
}
