﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopListImplement.Models;

namespace FlowerShopListImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataListSingleton _source;

        public OrderStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            var result = new List<OrderViewModel>();

            foreach (var order in _source.Orders)
            {
                result.Add(AttachInfo(order.GetViewModel));
            }

            return result;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            var result = new List<OrderViewModel>();

            if (model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.Id == model.Id || model.DateFrom <= order.DateCreate && order.DateCreate <= model.DateTo)
                    {
                        result.Add(AttachInfo(order.GetViewModel));
                    }
                }
            }

            if (model.ClientId.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.ClientId == model.ClientId)
                    {
                        result.Add(AttachInfo(order.GetViewModel));
                    }
                }
            }

            if (model.ImplementerId.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.ImplementerId == model.ImplementerId)
                    {
                        result.Add(AttachInfo(order.GetViewModel));
                    }
                }
            }

            if (model.Statuses != null)
            {
                foreach (var order in _source.Orders)
                {
                    if (model.Statuses.Contains(order.Status))
                    {
                        result.Add(AttachInfo(order.GetViewModel));
                    }
                }
            }

            return result;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            foreach (var order in _source.Orders)
            {
                if (model.Id.HasValue && order.Id == model.Id)
                {
                    return AttachInfo(order.GetViewModel);
                }
                else if (model.ImplementerId.HasValue && model.Statuses != null && order.ImplementerId == model.ImplementerId && model.Statuses.Contains(order.Status))
                {
                    return AttachInfo(order.GetViewModel);
                }
                else if (model.ImplementerId.HasValue && model.ImplementerId == order.ImplementerId)
                {
                    return AttachInfo(order.GetViewModel);
                }
            }

            return null;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = 1;
            foreach (var order in _source.Orders)
            {
                if (model.Id <= order.Id)
                {
                    model.Id = order.Id + 1;
                }
            }

            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }

            _source.Orders.Add(newOrder);

            return AttachInfo(newOrder.GetViewModel);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            foreach (var order in _source.Orders)
            {
                if (order.Id == model.Id)
                {
                    order.Update(model);
                    return AttachInfo(order.GetViewModel);
                }
            }

            return null;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            for (int i = 0; i < _source.Orders.Count; ++i)
            {
                if (_source.Orders[i].Id == model.Id)
                {
                    var element = _source.Orders[i];
                    _source.Orders.RemoveAt(i);
                    return AttachInfo(element.GetViewModel);
                }
            }

            return null;
        }

        private OrderViewModel AttachInfo(OrderViewModel model)
        {
            var bouquet = _source.Bouquets.Find(b => b.Id == model.BouquetId);
            model.BouquetName = bouquet is null ? String.Empty : bouquet.BouquetName;

            foreach (var client in _source.Clients)
            {
                if (client.Id == model.ClientId)
                {
                    model.ClientFIO = client.ClientFIO;
                    break;
                }
            }

            return model;
        }
    }
}
